/********************************************************
 * Data Visualization - 2D Data Visualization           *
 * Created in Processing 3                              *
 *                                                      *
 * Author: Alexander (Sandy) Schoettler                 *
 *                                                      *
 * Description: Demonstrate the changes in weekly       *
 *              checkout activity of books vs dvds      *
 *              in 2006 vs 2016. Each graph shows a     *
 *              heatmap of the number of checkouts      *
 *              for every hour (vertical) of every      *
 *              weekday (horizontal), aggregated over   *
 *              every week of the year for that item    *
 *              item type (book or dvd).                *
 *                                                      *
 * Usage:                                               *
 *          Press the number keys to apply different    *
 *          color schemes, or 0 for grayscale.          *
 ********************************************************/

import java.util.Arrays;
import processing.pdf.*;

void load_data(Table table, int[][] matrix) {
	for (TableRow row : table.rows()) {
		int tslice = row.getInt(0);
		for (int j=1; j < row.getColumnCount(); j++) {
			matrix[tslice][j-1] = row.getInt(j);
		}
	}
}

void print_matrix(int[][] matrix) {
		println("Matrix data: size [" + matrix.length + " x " + matrix[0].length + "]");
		for (int i=0; i < matrix.length; i++) {
				print("Row " + i + ":\t");
				for (int j=0; j < matrix[0].length; j++) {
						print("\t" + matrix[i][j]);
				}
				println();
		}
		println();
}

// Data variables
int verticalCells = 96; // 96 = number of 15-minute intervals in 24 hours = number of rows

// Layout variables
int constantHorizontalMargin = 80;
int constantVerticalMargin = 100;
int innerXMargin = 50;
int innerYMargin = 25;

int gridWidth;
int gridHeight;

int globalDataMaxValue;



private class Grid {
	// width and height for each cell
	float cellWidth;
	float cellHeight;

	int numRows;
	int numColumns;
	int startX, startY;


	public Grid(int startX, int startY) {
		this.startX = startX;
		this.startY = startY;
		this.numRows = verticalCells;
		this.numColumns = 7;
		this.cellWidth = gridWidth/this.numColumns;
		this.cellHeight = gridHeight/this.numRows;
	}
}

Grid g1, g2, g3, g4;

int[][] d1,d2,d3,d4;
int[][][] dataSets;


void load_datasets() {
	Table d1Table = loadTable("tslice15_2006_dvds.csv", "header");
	Table d2Table = loadTable("tslice15_2016_dvds.csv", "header");
	Table d3Table = loadTable("tslice15_2006_books.csv", "header");
	Table d4Table = loadTable("tslice15_2016_books.csv", "header");

	d1 = new int[verticalCells][7];
	d2 = new int[verticalCells][7];
	d3 = new int[verticalCells][7];
	d4 = new int[verticalCells][7];

	load_data(d1Table, d1);
	load_data(d2Table, d2);
	load_data(d3Table, d3);
	load_data(d4Table, d4);

	dataSets = new int[][][] {d1, d2, d3, d4};

}


// find maximum (absolute value) of a 2d matrix
int getMax(int[][] matrix) {
	int max = 0;
	for (int r = 0; r < matrix.length; r++) {
		for (int c = 0; c < matrix[0].length; c++) {
			if (abs(matrix[r][c]) > max) {
				max = abs(matrix[r][c]);
			}
		}
	}
	return max;
}

int getDataSetMax(int[][][] dsets) {
	int max = 0;
	for (int i = 0; i < dsets.length; ++i) {
		int dm = getMax(dsets[i]);
		if (dm > max) {
			max = dm;
		}
	}
	return max;
}

String[] days = {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"};

PImage colorBar;
String[] colorPath = {
	"Greys.png",
	"inferno.png",
	"magma.png",
	"plasma.png",
	"viridis.png",
	"RdPu.png",
	"GnBu.png",
	"YlGnBu.png",
	"bwReBlend129b.png",
	"cubehelix.png"};


boolean colorScheme = false;

void setup() {
	size(1920, 1080, PDF, "processing-output.pdf"); // setup the size of the window
	// In Processing 3, the size() function must be the first line of code inside setup()
	// and needs to use numbers instead of variables.


	// Make the screen resizable.
	surface.setResizable(true);

	gridWidth = (width - (2*constantHorizontalMargin + innerXMargin))/2;
	gridHeight = (height - (2*constantVerticalMargin + innerYMargin))/2;

	g1 = new Grid(
		constantHorizontalMargin,
		constantVerticalMargin);
	g2 = new Grid(
		constantHorizontalMargin + innerXMargin + gridWidth,
		constantVerticalMargin);
	g3 = new Grid(
		constantHorizontalMargin,
		constantVerticalMargin + innerYMargin + gridHeight);
	g4 = new Grid(
		constantHorizontalMargin + innerXMargin + gridWidth,
		constantVerticalMargin + innerYMargin + gridHeight);

	load_datasets();
	globalDataMaxValue = getDataSetMax(dataSets);
}

void drawGrid(Grid grid, int[][] data, String gridTitle, boolean axisLabelLeft) {
	for (int col = 0; col < grid.numColumns; col++) {
		for (int row = 0; row < grid.numRows; row++) {
			noStroke();
			int cellValue = data[row][col];

			if(!colorScheme){
				int greyScale = (int) (255 * cellValue / globalDataMaxValue);
				fill(greyScale);
			}
			else {
				color c = colorBar.get((int)map(cellValue, 0, globalDataMaxValue, 2, colorBar.width-2), colorBar.height/2);
				fill(c);
			}

			rect(grid.startX + grid.cellWidth*col,
					grid.startY + grid.cellHeight*row,
					grid.cellWidth,
					grid.cellHeight);
		}
	}

	// draw grid title
	textSize(16);
	textAlign(CENTER, TOP);
	fill(240);
	text(gridTitle, grid.startX + (gridWidth/2), grid.startY-45);

	// draw column headings
	textSize(12);
	textAlign(CENTER, TOP);
	fill(160);
	for (int day = 0; day < 7; day++) {
		text(days[day], grid.startX + (day*gridWidth/7) + (grid.cellWidth/2), grid.startY-19);
	}

	// draw time of day axes
	textSize(12);
	int xoffset = 0;
	if (axisLabelLeft) {
		textAlign(RIGHT, CENTER);
		xoffset = -8;
	}
	else {
		textAlign(LEFT, CENTER);
		xoffset = 8 + gridWidth;
	}
	fill(160);
	for (int hr = 4; hr < 24; hr += 4) {
		text(Integer.toString(hr), grid.startX + xoffset, grid.startY + (gridHeight * (hr-2)/24));
	}
}

void draw() {
	background(33);

	colorScheme = true;
	colorBar = loadImage(colorPath[1]);

	// Draw heatmaps
	drawGrid(g1, d1, "2006 DVDs", true);
	drawGrid(g2, d2, "2016 DVDs", false);
	drawGrid(g3, d3, "2006 Books", true);
	drawGrid(g4, d4, "2016 Books", false);

	//Draw Title
	textAlign(CENTER, CENTER);
	textSize(18);
	fill(240);
	text("WEEKLY BOOK AND DVD CHECKOUTS OVER A DECADE", g2.startX - 0.5 * innerXMargin, 30);

	exit();
}


void keyPressed() {
	if (Arrays.asList('0','1','2','3','4','5','6','7','8','9').contains(key)) {
		int num = (int)(key - '0');
		colorScheme = true;
		colorBar = loadImage(colorPath[num]);
	}

	if(key == 's' || key == 'S'){
		saveFrame();
	}
}